@@ -1,20 +1,6 @@
 /*************************************************************************
- * thinkfan version 0.7.3 -- copyleft 09-2011, Victor Mataré
+ * thinkfan version 0.8 -- copyleft 07-2011, Victor Mataré
  *
- * This file is part of thinkfan.
- *
- * A minimalist, lightweight fan control program for modern Thinkpads.
- *
- * This program is provided to you AS-IS. No warranties whatsoever.
- * If this program steals your car, kills your horse, smokes your dope
- * or pees on your carpet... too bad, you're on your own.
- *
- * Although it's generally considered bad style, I do make heavy use of
- * global variables, hoping to minimize memory access in the inner loop.
- * If you know better, please drop me a line. I haven't yet bothered looking
- * at the difference it makes in machine language.
-
-
  * thinkfan is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -27,6 +13,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with thinkfan.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
+ * Although it's generally considered bad style, I do make heavy use of
+ * global variables, hoping to minimize memory access in the inner loop.
+ * If you know better, please drop me a line. I haven't yet bothered looking
+ * at the difference it makes in machine language.
  *************************************************************************/
 #include "globaldefs.h"
 #include <string.h>
@@ -36,72 +28,109 @@
 #include <signal.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <time.h>
 #include "thinkfan.h"
 #include "message.h"
 #include "config.h" // The system interface is part of the config now...
 
+volatile int interrupted;
+unsigned int sleeptime, tmp_sleeptime;
+
+#define set_fan cur_lvl = config->limits[lvl_idx].level; \
+	if (!quiet && nodaemon) \
+	report(LOG_DEBUG, LOG_DEBUG, MSG_DBG_T_STAT); \
+	config->setfan();
+
+int run();
+
+/* Return TRUE if *at least one* upper limit has been reached */
+int complex_lvl_up() {
+	int i;
+
+	for (i=0; i < num_temps; i++)
+		if (temps[i] >= config->limits[lvl_idx].high[i]) return TRUE;
+	return FALSE;
+}
+
+/* Return TRUE if *all* lower limits have been reached */
+int complex_lvl_down() {
+	int i;
+
+	for (i=0; i < num_temps && temps[i] <= config->limits[lvl_idx].low[i]; i++);
+	if (i >= config->limit_len) return TRUE;
+	return FALSE;
+}
+
+int simple_lvl_up() {
+	if (unlikely(*b_tmax >= config->limits[lvl_idx].high[0])) return TRUE;
+	return FALSE;
+}
+
+int simple_lvl_down() {
+	if (unlikely(*b_tmax <= config->limits[lvl_idx].low[0])) return TRUE;
+	return FALSE;
+}
+
 
 /***********************************************************
  * This is the main routine which periodically checks
  * temperatures and adjusts the fan according to config.
  ***********************************************************/
 int fancontrol() {
-	int last_temp=0, temp, lvl_idx=0, bias=0, diff=0, b_temp;
-	int wt = watchdog_timeout, st = sleeptime;
+	int bias=0, diff=0, i;
+	int wt = watchdog_timeout;
+
+	tmp_sleeptime = sleeptime;
 
 	prefix = "\n"; // makes the output more readable
-	if(config->init_fan()) return ERR_FAN_INIT;
+	config->init_fan();
+	if (errcnt) return errcnt;
 
 	prefix = "\n"; // It is set to "" by the output macros
 
 	// Set initial fan level...
 	lvl_idx = config->num_limits - 1;
-	b_temp = temp = config->get_temp();
-	if (errcnt) return errcnt;
-	while ((temp <= config->limits[lvl_idx].low) \
-	 && (lvl_idx > 0)) lvl_idx--;
-	set_fan;
+
+	for (i=0; i < num_temps; i++)
+		if (temps[i] > tmax) tmax = temps[i];
 
 	/**********************************************
 	 * Main loop. This is the actual fan control.
 	 **********************************************/
 	while(likely(!interrupted && !errcnt)) {
-		last_temp = temp; // detect temperature jumps
-		sleep(st); // st is the state-dependant sleeptime
 
+		last_tmax = tmax;
 		// depending on the command line, this might also call depulse()
-		temp = config->get_temp();
-
-		// Write current fan level to IBM_FAN one cycle before the watchdog
-		// timeout ends, to let it know we're alive.
-		if (unlikely((wt -= st) <= sleeptime)) {
-#ifdef DEBUG
-			report(LOG_DEBUG, LOG_DEBUG, MSG_DBG_T_STAT);
-#endif
-			config->setfan();
-			wt = watchdog_timeout;
-		}
+		config->get_temps();
 
 		// If temperature increased by more than 2 °C since the
 		// last cycle, we try to react quickly.
-		diff = temp - last_temp;
+		diff = tmax - last_tmax;
 		if (unlikely(diff >= 2)) {
-			bias = (st * (diff-1)) * bias_level;
-			if (st > 2) st = 2;
+			bias = (tmp_sleeptime * (diff-1)) * bias_level;
+			if (tmp_sleeptime > 2) tmp_sleeptime = 2;
 		}
-		else if (unlikely(st < sleeptime)) st++;
-		b_temp = temp + bias;
+		else if (unlikely(tmp_sleeptime < sleeptime)) tmp_sleeptime++;
+		*b_tmax = tmax + bias;
 
-		if (unlikely(b_temp >= config->limits[lvl_idx].high)) {
-			while (likely((b_temp >= config->limits[lvl_idx].high) \
-			 && (lvl_idx < config->num_limits-1))) lvl_idx++;
+		// determine appropriate fan level and activate it
+		if (unlikely(lvl_idx < config->num_limits - 1 && config->lvl_up())) {
+			while (likely(lvl_idx < config->num_limits - 1 && config->lvl_up())) lvl_idx++;
 			set_fan;
 		}
-		if (unlikely(b_temp <= config->limits[lvl_idx].low)) {
-			while (likely((b_temp <= config->limits[lvl_idx].low) \
-			 && (lvl_idx > 0))) lvl_idx--;
+		else if (unlikely(lvl_idx > 0 && config->lvl_down())) {
+			while (likely(lvl_idx > 0 && config->lvl_down())) lvl_idx--;
 			set_fan;
-			st = sleeptime;
+			tmp_sleeptime = sleeptime;
+		}
+
+		sleep(tmp_sleeptime); // state-dependant sleeptime
+
+		// Write current fan level to IBM_FAN one cycle before the watchdog
+		// timeout ends, to let it know we're alive.
+		if (unlikely((wt -= tmp_sleeptime) <= sleeptime)) {
+			config->setfan();
+			wt = watchdog_timeout;
 		}
 
 		// slowly reduce the bias
@@ -148,11 +177,14 @@ int main(int argc, char **argv) {
 	nodaemon = 0;
 	errcnt = 0;
 	resume_is_safe = 0;
-	depulse = NULL;
 	prefix = "\n";
 	oldpwm = NULL;
-	cur_lvl = -1;
 	config = NULL;
+	lvl_idx = 0;
+	last_tmax = 0;
+	tmax = 0;
+	temps = NULL;
+	cur_lvl = NULL;
 
 	openlog("thinkfan", LOG_CONS, LOG_USER);
 	syslog(LOG_INFO, "thinkfan " VERSION " starting...");
@@ -222,9 +254,7 @@ int main(int argc, char **argv) {
 				}
 			}
 			else depulse_tmp = 0.5f;
-			depulse = (struct timespec *) malloc(sizeof(struct timespec));
-			depulse->tv_sec = (time_t)depulse_tmp;
-			depulse->tv_nsec = 1000*1000*1000 * (depulse_tmp - depulse->tv_sec);
+			depulse = (useconds_t) (1000*1000 * depulse_tmp);
 			break;
 		default:
 			fprintf(stderr, MSG_USAGE);
@@ -255,8 +285,8 @@ int main(int argc, char **argv) {
 fail:
 	free_config(config);
 	free(rbuf);
-	free(depulse);
 	free(oldpwm);
+	free(temps);
 	return ret;
 }
 
@@ -276,8 +306,10 @@ int run() {
 		report(LOG_ERR, LOG_ERR, MSG_ERR_CONF_NOFILE);
 		return ERR_CONF_NOFILE;
 	}
-	if (config->init_fan()) return ERR_FAN_INIT;
-	if (config->get_temp() == ERR_T_GET) return ERR_T_GET;
+
+	config->init_fan();
+	config->get_temps();
+	if (errcnt) return errcnt;
 
 	if (chk_sanity && ((pidfile = fopen(PID_FILE, "r")) != NULL)) {
 		fclose(pidfile);
@@ -300,7 +332,7 @@ int run() {
 		}
 	}
 
-	if ((pidfile = fopen(PID_FILE, "w+")) == NULL) {
+	if ((pidfile = fopen(PID_FILE, "w+")) == NULL && chk_sanity) {
 		report(LOG_ERR, LOG_WARNING, PID_FILE ": %s", strerror(errno));
 		return ERR_PIDFILE;
 	}
